&nbsp;

<p align="center">
  <a href="https://steinhq.com">
    <img src="https://steinhq.com/static/media/Logo.7a44b5e9.svg" alt="Stein" />
  </a>
</p>

<h3 align="center">Ship fast and manage your data with ease. Connect to Google Sheets.</h3>

<p align="center">
	Kick off projects, design custom Google Forms, and manage your content in a familiar interface. 
	<br>
	Power your project with the fully open source Stein.
</p>

<p align="center" style="font-size:0.75rem">
	<a href="https://steinhq.com">SteinHQ.com</a> | 
	<a href="https://docs.steinhq.com">Documentation</a>
</p>

## Setup

There are two ways you can get started with the Stein API:

- Use the [hosted service](https://steinhq.com) to get a free & reliable API in a couple of clicks.
- Self-host an instance of Stein. Find the related documentation [here](https://docs.steinhq.com/hosting-introduction).

## Examples

Details and examples on the complete functionality can be found in the [documentation](https://docs.steinhq.com). Here are a few to get you started!

All the data and the errors are communicated using JSON. You may perform common operations on your sheets such as read, search, write, etc.

Structure your sheet as shown below, with the first row populated with column names.

<p align="center">
	<img src="https://docs.steinhq.com/assets/prepared-sheet.png" alt="Sheet Structure" />
</p>

A read operation on the sheet will return an array of the rows.

```
[
  {
    "title":"Why the Best Things in Life Can’t Be Planned",
    "content":"Thales of Miletus, considered ...",
    "link":"https://medium.com/...",
    "author":"Zat Rana"
  },
  ...
]
```

### Using the core API

Since Stein is a REST API, there are no limitations as to which languages you can use. For this example, let's use the [Stein JavaScript Client](https://github.com/SteinHQ/JS-Client/) to obtain the data:

```javascript
const SteinStore = require("stein-js-client");

// Instantiate store for spreadsheet API URL
const store = new SteinStore(
  "https://api.steinhq.com/v1/storages/5cc158079ec99a2f484dcb40"
);

// Read Sheet1 of spreadsheet
store.read("Sheet1").then(data => {
  console.log(data);
});

// Logs object like ↓
// [{title:"Why the Best Things in Life Can’t Be Planned",content:"Thales of Miletus, considered ...",link:"https://medium.com/...",author:"Zat Rana"}, {...}, ...]
```

### Using plain HTML

To simply display the data on a webpage, we don't even need JS! Using [Stein Expedite](https://docs.steinhq.com/expedite-introduction),

```html
<!--- Replace the data-stein-url value with your API URL --->
<div
  data-stein-url="http://api.steinhq.com/v1/storages/5cc158079ec99a2f484dcb40/Sheet1"
>
  <div>
    <h1>{{title}}</h1>
    <h6>By {{author}}</h6>
    <p>
      {{content}}
    </p>
    <p>Read on <a href="{{link}}">Medium</a></p>
  </div>
</div>
```

Here's a minimal output of the above code.

<p align="center">
	<img src="https://docs.steinhq.com/assets/expedite-blog-posts-read.png" alt="Expedite Output" />
</p>

## Contributing

Stein is completely open-source software, and the best part about structuring it this way is that everyone gets to own, understand, and improve it.

The main purpose of this repository is to continue to evolve the Stein Core API, making it faster and easier to use. We are grateful to the community for contributing fixes and improvements.

### [Code Of Conduct](./CODE_OF_CONDUCT.md)

All participants are expected to adhere to the Code of Conduct.

### [Contributing Guidelines](./CONTRIBUTING.md)

Read our contributing guide to learn about what contributions we are looking for and how to propose them.

## Built With

- [Node.js](https://github.com/nodejs/node) + [Express](https://github.com/expressjs/express): The back-end API is an Express app. It responds to requests RESTfully in JSON.
- [MongoDB](https://github.com/mongodb/mongo): The main data store.

## Partners

Stein officially partners with the following companies, and thanks them for their support!

<p>
  <a href="https://www.digitalocean.com/">
    <img src="https://opensource.nyc3.cdn.digitaloceanspaces.com/attribution/assets/SVG/DO_Logo_horizontal_blue.svg" height="32px">
  </a>
    
  <a href="https://www.gandi.net/">
    <img src="https://v4.gandi.net/static/images/illustrations/press/Gandi_logo_black.jpg" height="32px">
  </a>
</p>

For any queries regarding partnerships, reach out to [SteinHQ](mailto:hello@steinhq.com).

## License

The Stein core project is [licensed under GNU GPLv3](./LICENSE).

To distribute commercially licensed software with Stein and not distribute the source code for the commercially licensed software under version 3 of the GNU General Public License (the "GPL"), contact [SteinHQ](mailto:hello@steinhq.com).
